local addon = WakeSpams
local module = addon:NewModule("SpellDataBase")
WakeSpams.sdbm = module

local function InjectSpellDB(tbl)
	local version = addon.db.profile.version
	local spelldb = addon.db.profile.spelldb[UnitGUID("player")]
	
	for t,k in pairs(tbl) do
		if(spelldb[t]) then
			
			if(k.version.spell) and (k.version.spell > version) then
				k.version = nil
				spelldb[t] = k
			else
				for key,value in pairs(k) do
					if(k.version[key]) and (k.version[key] > version) then
						spelldb[t][key] = value
					end
				end
			end
			
		else
			k.version = nil
			spelldb[t] = k
		end
	end
	
end

function module:InitSpellDB()
	local _, class = UnitClass("player");
	assert(class, "Error: WakeSpams was unable to retrieve player englishclass.")
	
	local PLAYER = UnitGUID("player")
	
	if not addon.db.profile.spelldb[PLAYER] then
		addon.db.profile.spelldb[PLAYER] = {}
	end
	
	--[[
		SpellList args:
		Spell ID {
			disabled = boolean
			name = string (optional) custom spell name
			txt = string (optional) message on start
			txtend = string (optional) message on end8
			countdown = int (optional) fade countdown in chat
			fade = boolean (optional) announce when spell fades
			whisper = boolean
			output = string (optional) output channel
			event = string (optional) event name
			title = string (optional) name that will show up in the config
			
			hastarget = boolean  
			hasextra = boolean  idklol
			isaura = boolean  check duration
			isaoe = boolean  perform aoe checks
			
			fademode = int (optional) 1 = check other spell. Currently: Other spell must be present as DB spell.
										  If other spell fires, it will use message FROM THAT SPELL. Fade message from original spell wont fire. This is cheap, but easy.
									  2 = Check previous spell
			[	fadearg = str/int/bool - If fademode = 1, then fadearg must be spell ID (int)]
		}
	]]
	local spelldb = {}
	
	if(class == "DEATHKNIGHT") then
		spelldb = {
			[48792] = { -- Icebound Fortitude
				dur = 12,
				isaura = true,
				fade = true,
				countdown = 0,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[55233] = { -- Vampiric Blood
				dur = 10,
				isaura = true,
				countdown = 0,
				fade = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[56222] = { -- Dark Command taunt
				txt = "Taunt Failed: spell:link on spell:target (spell:extra)",
				hastarget = true,
				events = {
					["SPELL_MISSED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[49560] = { -- Death Grip (taunt component)
				txt = "Taunt Failed: spell:link on spell:target (spell:extra)",
				hastarget = true,
				events = {
					["SPELL_MISSED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[49016] = { -- Unholy Frenzy (whisper)
				txt = "Activated: spell:link on spell:target - spell:durations!",
				hastarget = true,
				dur = 30,
				isaura = true,
				whisper = true,
				countdown = 0,
				fade = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[47476] = { -- Strangulate
				txt = "spell:link: spell:target",
				hastarget = true,
				isaura = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
		}
	end
	
	if(class == "DRUID") then
		spelldb = {
			[22812] = { -- Barkskin
				dur = 12,
				isaura = true,
				countdown = 0,
				fade = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[61336] = { -- Survival Instincts
				dur = 12,
				isaura = true,
				countdown = 0,
				fade = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[29166] = { -- Innervate (whisper)
				txt = "Activated: spell:link on spell:target - spell:durations!",
				hastarget = true,
				dur = 10,
				isaura = true,
				countdown = 0,
				fade = true,
				whisper = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[5211] = { -- Bash
				isaura = true,
				txt = "Activated: spell:link on spell:target!",
				hastarget = true,
				countdown = 0,
				fade = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[20484] = { -- Rebirth
				txt = "spell:link on spell:target!",
				hastarget = true,
				events = {
					["SPELL_RESURRECT"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[50769] = { -- Revive
				txt = "Casting: spell:link on spell:target",
				hastarget = true,
				events = {
					["UNIT_SPELLCAST_START"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
		}
	end
	
	if(class == "HUNTER") then
		spelldb = {
			[34477] = { -- Misdirection (cast... Not tested)
				txt = "spell:link on spell:target!",
				hastarget = true,
				events = {
					["SPELL_CAST_SUCCESS"] = true,
				},
				whisper = true,
				version = {
					spell = 4103002,
				},
			},
			-- Pet Abilities
			[90355] = { -- Ancient Hysteria
				txt = "Activated: spell:link spell:durations!",
				dur = 40,
				isaura = true,
				countdown = 0,
				aoe = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[53480] = { -- Roar of Sacrifice
				dur = 12,
				isaura = true,
				countdown = 0,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
		}
	end
	
	if(class == "MAGE") then
		spelldb = {
			[80353] = { -- Time warp
				txt = "Activated: spell:link spell:durations!",
				dur = 40,
				isaura = true,
				countdown = 0,
				aoe = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
		}
	end
	
	if(class == "PALADIN") then
		spelldb = {
			--[[
			[20233] = { -- Lay on Hands (holy phys. damage reduction only. Right spell id?)
				txt = "Activated: spell:link on spell:target - spell:durations!",
				dur = 15,
				countdown = 0,
				fade = true,
			},]]
			[31821] = { -- Aura Mastery
				dur = 6,
				isaura = true,
				countdown = 0,
				fade = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[498] = { -- Divine Protection
				dur = 10,
				isaura = true,
				countdown = 0,
				fade = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[70940] = { -- Divine Guardian
				dur = 6,
				isaura = true,
				countdown = 0,
				fade = true,
				aoe = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[66235] = { -- Ardent Defender proc
				txt = "spell:link saved me! Healed spell:amount!",
				events = {
					["SPELL_HEAL"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[853] = { -- Hammer of Justice
				isaura = true,
				txt = "Activated: spell:link on spell:target!",
				hastarget = true,
				countdown = 0,
				fade = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[2812] = { -- Holy Wrath
				dur = 3,
				isaura = true,
				countdown = 0,
				fade = true,
				aoe = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[1044] = { -- Hand of Freedom
				txt = "spell:link on spell:target - spell:durations!",
				hastarget = true,
				dur = 6,
				isaura = true,
				whisper = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[1022] = { -- Hand of Protection
				txt = "Activated: spell:link on spell:target - spell:durations!",
				hastarget = true,
				dur = 10,
				isaura = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[6940] = { -- Hand of Sacrifice
				txt = "Activated: spell:link on spell:target - spell:durations!",
				hastarget = true,
				dur = 12,
				isaura = true,
				countdown = 0,
				fade = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[1038] = { -- Hand of Salvation (whisper)
				dur = 10,
				isaura = true,
				txt = "Activated: spell:link on spell:target - spell:durations!",
				hastarget = true,
				countdown = 0,
				fade = true,
				whisper = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[7328] = { -- Redemption
				txt = "Casting: spell:link on spell:target",
				hastarget = true,
				events = {
					["UNIT_SPELLCAST_START"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
		}
	end
	
	if(class == "PRIEST") then
		spelldb = {
			[65081] = { -- Body and Soul (target effect triggered by body and soul)
				txt = "spell:link on spell:target - meep meep!",
				hastarget = true,
				dur = 4,
				isaura = true,
				whisper = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[8122] = { -- Psychic Scream
				dur = 8,
				isaura = true,
				aoe = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[10060] = { -- Power Infusion (whisper)
				txt = "Activated: spell:link on spell:target - spell:durations!",
				hastarget = true,
				dur = 15,
				isaura = true,
				whisper = true,
				fade = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[64901] = { -- Hymn of Hope
				txt = "Activated: spell:link!",
				events = {
					["SPELL_CAST_SUCCESS"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[47788] = { -- Guardian Spirit
				txt = "Activated: spell:link on spell:target - spell:durations!",
				txtend = "Faded (normal): spell:link from spell:target",
				hastarget = true,
				dur = 10,
				isaura = true,
				countdown = 2,
				fade = true,
				advfade = "1",
				fademode = 1,
				fadearg = 48153,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[48153] = { -- Guardian Spirit (heal)
				txt = "Faded (PROCCED): spell:link healed spell:target for spell:amount",
				hastarget = true,
				events = {
					["SPELL_HEAL"] = true,
				},
				title = "Guardian Spirit (heal effect)",
				fademode = 2,
				fadearg = 47788,
				version = {
					spell = 4103002,
				},
			},
			[33206] = { -- Pain Suppression
				txt = "Activated: spell:link on spell:target - spell:durations!",
				hastarget = true,
				dur = 8,
				isaura = true,
				fade = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[64843] = { -- Divine Hymn
				txt = "Activated: spell:link!",
				hastarget = true,
				events = {
					["SPELL_CAST_SUCCESS"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[2006] = { -- Resurrection
				txt = "Casting: spell:link on spell:target",
				hastarget = true,
				events = {
					["SPELL_RESURRECT"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
		}
	end
	
	if(class == "ROGUE") then
		spelldb = {
			[57934] = { -- Tricks of the Trade (cast... Not tested)
				txt = "spell:link on spell:target!",
				hastarget = true,
				dur = 6,
				events = {
					["SPELL_CAST_SUCCESS"] = true,
				},
				whisper = true,
				version = {
					spell = 4103002,
				},
			},
			[1833] = { -- Cheap Shot
				txt = "Activated: spell:link on spell:target!",
				hastarget = true,
				countdown = 0,
				fade = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[408] = { -- Kidney Shot
				txt = "Activated: spell:link on spell:target!",
				hastarget = true,
				countdown = 0,
				fade = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
		}
	end
	
	if(class == "SHAMAN") then
		spelldb = {
			[2825] = { -- Bloodlust
				txt = "Activated: spell:link spell:durations!",
				dur = 40,
				countdown = 0,
				aoe = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[32182] = { -- Heroism
				txt = "Activated: spell:link spell:durations!",
				hastarget = true,
				dur = 40,
				countdown = 0,
				aoe = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[2008] = { -- Ancestral Spirit
				txt = "Casting: spell:link on spell:target",
				hastarget = true,
				events = {
					["SPELL_RESURRECT"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
		}
	end
	
	if(class == "WARRIOR") then
		spelldb = {
			[871] = { -- Shield Wall
				dur = 12,
				countdown = 0,
				fade = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[12976] = { -- Last Stand
				dur = 20,
				countdown = 0,
				fade = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[46968] = { -- Shockwave
				dur = 4,
				countdown = 0,
				fade = true,
				aoe = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[12809] = { -- Concussion Blow
				txt = "Activated: spell:link on spell:target!",
				hastarget = true,
				dur = 5,
				countdown = 0,
				fade = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
		}
	end
	
	if(class == "WARLOCK") then
		spelldb = {
			[1122] = { -- Summon Inferno
				txt = "Activated: spell:link!",
				countdown = 0,
				fade = true,
				events = {
					["SPELL_CAST_SUCCESS"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
			[5484] = { -- Howl of Terror
				dur = 8,
				aoe = true,
				events = {
					["SPELL_AURA_APPLIED"] = true,
					["SPELL_AURA_REMOVED"] = true,
				},
				version = {
					spell = 4103002,
				},
			},
		}
	end
	
	InjectSpellDB(spelldb)
end